<?php

namespace Drupal\pdf_using_mpdf\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountSwitcherInterface;
use Drupal\Core\Session\AnonymousUserSession;
use Drupal\pdf_using_mpdf\Conversion\ConvertToPdf;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * PDF generator.
 */
class GeneratePdf extends ControllerBase {

  /**
   * Entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Drupal renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Convert to PDF service.
   *
   * @var \Drupal\pdf_using_mpdf\Conversion\ConvertToPdf
   */
  protected $convertToPdf;

  /**
   * The account switcher service.
   *
   * @var \Drupal\Core\Session\AccountSwitcherInterface
   */
  protected $accountSwitcher;

  /**
   * Inject ConvertToPdf service.
   *
   * GeneratePdf constructor.
   *
   * @param \Drupal\pdf_using_mpdf\Conversion\ConvertToPdf $convert
   *   Converter definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   EntityTypeManager definition.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   Module handler definition.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Session\AccountSwitcherInterface $accountSwitcher
   *   The account switcher service.
   */
  public function __construct(ConvertToPdf $convert, EntityTypeManagerInterface $entityTypeManager, ModuleHandlerInterface $moduleHandler, RendererInterface $renderer, AccountSwitcherInterface $accountSwitcher) {
    $this->convertToPdf = $convert;
    $this->entityTypeManager = $entityTypeManager;
    $this->moduleHandler = $moduleHandler;
    $this->renderer = $renderer;
    $this->accountSwitcher = $accountSwitcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('pdf_using_mpdf.conversion'),
      $container->get('entity_type.manager'),
      $container->get('module_handler'),
      $container->get('renderer'),
      $container->get('account_switcher'),
    );
  }

  /**
   * Generate a PDF for a given node.
   *
   * @param \Drupal\node\NodeInterface|null $node
   *   Processed node.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirect response.
   */
  public function generate($node = NULL) {

    // Get settings.
    $settings = $this->convertToPdf
      ->getConfig();

    // Let other modules alter and overwrite mPDF settings.
    $this->moduleHandler
      ->alter('mpdf_settings', $settings, $node);

    // Get view mode.
    $view_mode = $settings['view_mode'] ?? 'full';

    // Check for anonymous rendering.
    if ($settings['render_anonymous']) {
      $this->accountSwitcher->switchTo(new AnonymousUserSession());
    }

    try {
      // Load and render node.
      $view = $this->entityTypeManager
        ->getViewBuilder('node')
        ->view($node, $view_mode);

      $renderedNode = $this->renderer
        ->render($view)->__toString();
    }
    finally {
      // Check for anonymous rendering.
      if ($settings['render_anonymous']) {
        $this->accountSwitcher->switchBack();
      }
    }

    // Let other modules alter HTML for PDF generation.
    $this->moduleHandler
      ->alter('mpdf_html', $renderedNode, $node);

    return $this->convertToPdf
      ->convert($renderedNode, $settings, ['node' => $node]);
  }

}
